/// Josh's 3D Math Library
/// A C++20 Library for 3D Math, Computer Graphics, and Scientific Computing.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file Matrix2x2.hpp
/// @desc A two-by-two Matrix object.
/// @edit 2024-08-01
/// @note On backlog, low-priority.

#pragma once



#include <J3ML/LinearAlgebra.hpp>

namespace J3ML::LinearAlgebra {
    class Matrix2x2 {
    public:
        enum { Rows = 2 };
        enum { Cols = 2 };
        static const Matrix2x2 Zero;
        static const Matrix2x2 Identity;
        static const Matrix2x2 NaN;

        Matrix2x2() {}
        Matrix2x2(float val);
        Matrix2x2(float m00, float m01, float m10, float m11);
        Matrix2x2(const Vector2& r1, const Vector2& r2);
        explicit Matrix2x2(const float *data);

        Vector2 GetRow(int index) const;
        Vector2 GetColumn(int index) const;

        void SetRow(int i, const Vector2& row);
        void SetColumn(int i, const Vector2& col);
        void SetAt(int x, int y, float value);

        float At(int x, int y) const;
        float &At(int x, int y);

        float Determinant() const;
        Matrix2x2 Inverse() const;
        Matrix2x2 Transpose() const;

        Vector2 Transform(const Vector2& rhs) const;


        Vector2 operator * (const Vector2& rhs) const;
        Matrix2x2 operator * (const Matrix2x2 &rhs) const;

        inline float* ptr() { return &elems[0][0];}
        [[nodiscard]] inline const float* ptr() const {return &elems[0][0];}

    protected:
        float elems[2][2];
    };
}